#!/usr/bin/python3
#
# Download and install Storage Made Easy File Fabric software
#
# Author: Storage Made Easy <support@storagemadeeasy.com>

from charms.reactive import when, when_not, set_state, only_once

from subprocess import check_call, call

from charmhelpers.core.hookenv import status_set, Hooks
from charmhelpers.core.host import add_group
from charmhelpers.core.host import adduser
from charmhelpers.core.hookenv import open_port
from charmhelpers.fetch import apt_install
from charmhelpers.fetch import apt_update
import charms.leadership
#from charmhelpers.core.templating import render
import os
import sys
import fileinput
import pwd

@when('apt.installed.cpanminus')
@when_not('sme.cpan_installed')
def install_cpan():
    status_set('maintenance', 'Installing CPAN Packages')
    cpan_packages = [
        'Test::Simple',
        'Test::Warn',
        'Test::NoWarnings',
        'Class::Accessor',
        'XML::Parser::EasyTree',
        'CGI::Ajax',
        'Date::Format',
        'Date::Parse',
        'Date::Manip',
        'IO::Tty'
    ]

    for i in cpan_packages:
        status_set('maintenance', 'Installing CPAN Package: {}'.format(i))
        check_call(['cpanm', i])

    set_state('sme.cpan_installed')
    status_set('maintenance', 'CPAN Packages installed')

@when('sme.cpan_installed')
@when_not('sme.groups_added')
def add_groups():
    status_set('maintenance', 'Creating system groups')

    groups = [
        'smestorage',
        'smeconfiguser',
        'defaultstore',
        'sme'
    ]

    # This is a comment, you can delete me
    # If you find yourself using subprocess a lot, odds are there is a helper method
    # in charmhelpers for that. The documentation can be a bit unwieldy, but check out
    # http://pythonhosted.org/charmhelpers/api/charmhelpers.html and use the search
    # to try to map to things you're subprocessing for
    for group in groups:
        add_group(group)
    status_set('maintenance', 'System groups created')
    set_state('sme.groups_added')


@when('sme.groups_added')
@when_not('sme.users_ready')
def add_users():
    status_set('maintenance', 'Creating user accounts')
    # This is a comment, you can delete me
    # There is a charmhelpers.core.host.adduser method however you can not set the HOME
    # For the created user. This needs to be requested / patched in the charmhelpers library
    # http://pythonhosted.org/charmhelpers/api/charmhelpers.core.host.html#charmhelpers.core.host.adduser
    try:
        pwd.getpwnam('smestorage')
    except KeyError:
        check_call(['useradd', '-m', '-p', '$6$mQM0S5fe$01X38ZTeOBEHkVovBYJ2Qn2NQKHdVRkpQmHItCx/lkTfNUZ8ge2v/aYHIm9y8LcxvjeMmzcHdFd0Y48B25NSq/','-s', '/bin/bash', '-g', '1001', '-d', '/var/www/smestorage', 'smestorage' ])

    try:
        pwd.getpwnam('smeconfiguser')
    except KeyError:
        check_call(['useradd', '-m', '-p', '$6$hqEj9dSs$GuPJztaqvlbcWR6DQO8H8Y1qh9DMsuKlcqonoltcIWViewQZx2FXiYxayFAmM6cUvfhjOTIv/6B9XJ73kzBjn1','-s', '/bin/bash', '-g', '1002', 'smeconfiguser'])

    try:
        pwd.getpwnam('defaultstore')
    except KeyError:
        check_call(['useradd', '-m', '-p', '$6$mxFL7iuJFW$HngqnJxnJi76OZ2/VLDGWReCxpfKJj5U5FVaAt7W4tTYTc2i9YIZ1x9flbCR91tzJt5wHgMI5P2kSx.UAxpUN0','-s', '/bin/bash', '-g', '1003', 'defaultstore'])

    try:
        pwd.getpwnam('sme')
    except KeyError:
        check_call(['useradd', '-m','-s','/bin/bash', '-g', '1004', 'sme'])

    set_state('sme.users_ready')


@when('apt.installed.language-pack-en-base')
@when('sme.users_ready')
@when_not('sme.php_installed')
def install_php():
    status_set('maintenance', 'Installing PHP5.6')
    os.environ['LC_ALL']='en_US.UTF-8'
    os.environ['LANG']='en_US.UTF-8'
    check_call(['apt-add-repository', 'ppa:ondrej/php', '-y'])
    apt_update()

    php_packages = [
      'php5.6-cli',
      'php5.6-mysql',
      'php5.6-zip',
      'php5.6-mbstring',
      'php5.6-gd',
      'php5.6',
      'php5.6-fpm',
      'php5.6-curl',
      'php5.6-xml',
      'php-pear',
      'php-memcache',
      'php5.6-imap',
      'php5.6-ldap',
      'php5.6-mcrypt',
      'php5.6-soap',
      'php5.6-xmlrpc']

    for package in php_packages:
        apt_install(package)

    set_state('sme.php_installed')
    status_set('maintenance', 'PHP has been installed')
#    set_state('sme.ready')




@when('sme.php_installed')
@when_not('sme.jetty_configured')
@only_once
def configure_jetty():
    status_set('maintenance', 'Configuring Jetty')
    # SME
    check_call(['wget', '-q', 'https://repo1.maven.org/maven2/org/eclipse/jetty/jetty-distribution/9.1.0.v20131115/jetty-distribution-9.1.0.v20131115.tar.gz', '-P', '/home/sme'])
    check_call(['tar', 'zxf', '/home/sme/jetty-distribution-9.1.0.v20131115.tar.gz', '-C', '/home/sme'])
    os.symlink('/home/sme/jetty-distribution-9.1.0.v20131115', '/home/sme/jetty-latest')
    check_call(['wget', '-q', 'https://archive.apache.org/dist/lucene/solr/4.6.0/solr-4.6.0.tgz', '-P', '/home/sme'])
    check_call(['tar', 'zxf', '/home/sme/solr-4.6.0.tgz', '-C', '/home/sme'])
    os.symlink('/home/sme/solr-4.6.0', '/home/sme/solr-latest')
    check_call(['mkdir', '/home/sme/sme_jetty'])
    java_cmd = 'cd /home/sme/sme_jetty/; java -jar /home/sme/jetty-distribution-9.1.0.v20131115/start.jar --add-to-start=resources,ext,http,deploy,jsp,logging,jmx,jmx-remote'
    check_call(java_cmd, shell=True)
    os.mkdir('/home/sme/sme_jetty/etc')
    check_call(['cp', '/home/sme/solr-4.6.0/example/webapps/solr.war', '/home/sme/sme_jetty/webapps/'])
    check_call(['wget', '-q', 'https://s3.amazonaws.com/smevm/juju-charms/config/jetty.sh', '-P', '/home/sme/sme_jetty/'])

# INIT
    check_call(['wget', '-q', 'https://s3.amazonaws.com/smevm/juju-charms/config/init/jetty', '-O', '/etc/init.d/jetty'])
    check_call(['wget', '-q', 'https://s3.amazonaws.com/smevm/juju-charms/config/default/jetty', '-O', '/etc/default/jetty'])
    os.chmod('/etc/init.d/jetty', 0o755)

    check_call(['wget', '-q', 'https://s3.amazonaws.com/smevm/juju-charms/config/realm.properties', '-P', '/home/sme/sme_jetty/etc/'])
    check_call(['wget', '-q', 'https://s3.amazonaws.com/smevm/juju-charms/config/webdefault.xml', '-P', '/home/sme/sme_jetty/etc/'])
    check_call(['wget', '-q', 'https://s3.amazonaws.com/smevm/juju-charms/config/start.ini', '-O', '/home/sme/sme_jetty/start.ini'])
    check_call(['wget', '-q', 'https://s3.amazonaws.com/smevm/juju-charms/config/log4j.properties', '-P', '/home/sme/sme_jetty/resources/'])
    check_call(['wget', '-q', 'https://s3.amazonaws.com/smevm/juju-charms/config/solr.xml', '-P', '/home/sme/sme_jetty/webapps/'])
    check_call(['wget', '-q', 'https://s3.amazonaws.com/smevm/juju-charms/config/pdfburner.war', '-P', '/home/sme/sme_jetty/webapps/'])
    check_call(['wget', '-q', 'https://s3.amazonaws.com/smevm/juju-charms/config/jetty.xml', '-P', '/home/sme/sme_jetty/etc/'])

    check_call(['wget', '-q', 'http://central.maven.org/maven2/org/slf4j/jcl-over-slf4j/1.6.6/jcl-over-slf4j-1.6.6.jar', '-P', '/home/sme/sme_jetty/lib/ext/'])
    check_call(['wget', '-q', 'http://central.maven.org/maven2/org/slf4j/jul-to-slf4j/1.6.6/jul-to-slf4j-1.6.6.jar', '-P', '/home/sme/sme_jetty/lib/ext/'])
    check_call(['wget', '-q', 'http://central.maven.org/maven2/log4j/log4j/1.2.16/log4j-1.2.16.jar', '-P', '/home/sme/sme_jetty/lib/ext/'])
    check_call(['wget', '-q', 'http://central.maven.org/maven2/org/slf4j/slf4j-api/1.6.6/slf4j-api-1.6.6.jar', '-P', '/home/sme/sme_jetty/lib/ext/'])
    check_call(['wget', '-q', 'http://central.maven.org/maven2/org/slf4j/slf4j-log4j12/1.6.6/slf4j-log4j12-1.6.6.jar', '-P', '/home/sme/sme_jetty/lib/ext/'])

    os.mkdir('/home/sme/sme_jetty/temp')

    # SMEDATA
    os.mkdir('/smedata')
    os.mkdir('/smedata/pdfburn')
    check_call(['wget', '-q', 'https://smevm.s3.amazonaws.com/juju-charms/sme_solr.tar.gz', '-P', '/tmp'])
    check_call(['tar', 'zxf', '/tmp/sme_solr.tar.gz', '-C', '/smedata'])

    os.chmod('/home/sme/sme_jetty/jetty.sh', 0o755)
    check_call(['chown', '-R', 'sme:sme', '/home/sme'])
    check_call(['chown', '-R', 'sme:sme', '/smedata'])
    check_call(['chown', '-R', 'smestorage:sme', '/smedata/pdfburn'])
    os.chmod('/smedata/pdfburn', 0o770)
    check_call(['systemctl', 'enable', 'jetty'])
    check_call(['systemctl', 'start', 'jetty'])

    status_set('maintenance', 'Jetty configured')
    set_state('sme.jetty_configured')

@when('sme.jetty_configured')
@when_not('sme.http_mods_enabled')
def enable_apache_mods():
    status_set('maintenance', 'Enabling Apache2 modules')
    check_call('/usr/sbin/a2enmod rewrite',shell=True)
    check_call('/usr/sbin/a2enmod suexec',shell=True)
    check_call('/usr/sbin/a2enmod cgi',shell=True)
    check_call('/usr/sbin/a2enmod proxy_fcgi',shell=True)
    check_call('/usr/sbin/a2enmod ssl',shell=True)
    check_call('/usr/sbin/a2enmod headers',shell=True)
    check_call(['a2dissite', '000-default'])
    check_call(['mkdir', '/var/lib/php/smestorage/'])
    check_call(['chown', 'smestorage:smestorage', '/var/lib/php/smestorage/'])
    set_state('sme.http_mods_enabled')

@when('sme.http_mods_enabled')
@when_not('sme.apache_configs')
def copy_apache_configs():
    status_set('maintenance', 'Adding Apache2 Vhost')

    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/appliancebuild/conf/apache2/sme.conf', '-O', '/etc/apache2/sites-enabled/sme.conf'])
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/appliancebuild/conf/apache2/sme-ssl.conf', '-O', '/etc/apache2/sites-enabled/sme-ssl.conf'])

    set_state('sme.apache_configs')


@when('sme.apache_configs')
@when_not('sme.cloudftp_init')
def cloudftp_init():
    status_set('maintenance', 'Adding CloudFTP init.d')
    vsftp_init = """
#!/usr/bin/env bash
### BEGIN INIT INFO
# Provides:          cloudftp
# Required-Start:    $network
# Required-Stop:     $network
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: starts cloudftp
# Description:       Starts cloudftp
### END INIT INFO

case "$1" in
start)
echo "starting cloud ftp.."
pwd
cd /var/www/smestorage/ftpserver
./smestorage_ftp_server.pl
echo "starting cloud ftps.."
./smestorage_ftp_server.pl -i
echo "starting sftp"
su - smestorage -c "cd /var/www/smestorage/ftpserver/sftpserver/; /usr/bin/python2.7 sme_sftp.py --restart"

cd /
;;

stop)
echo "stopping cloud ftp.."
ps -ef | grep "/usr/bin/perl ./smestorage_ftp_server.pl" | grep -v grep | awk '{print $2}' |xargs kill -9
pgrep -f sme_sftp.py|xargs kill -9
;;

restart)
        $0 stop
        $0 start
;;

*)
        echo "Usage: $0 {start|stop|restart}"
        exit 1
esac

    """
    with open('/etc/init.d/cloudftp', 'w') as out:
        out.write(vsftp_init)
    os.chmod('/etc/init.d/cloudftp', 0o755)
    check_call(['systemctl', 'restart', 'vsftpd'])
    check_call(['systemctl', 'enable', 'cloudftp'])
    #check_call(['systemctl', 'start', 'cloudftp'])
    set_state('sme.cloudftp_init')

@when('sme.cloudftp_init')
@when_not('sme.vsftpd_init')
def vsftpd_config():
    status_set('maintenance', 'Configuring vsftpd')
    if not os.path.exists('/etc/vsftpd'):
        os.mkdir('/etc/vsftpd')

    os.remove('/etc/vsftpd.conf')
    os.symlink('/etc/vsftpd/vsftpd.conf', '/etc/vsftpd.conf')
    vsftp_config = """anonymous_enable=NO
local_enable=YES
write_enable=YES
local_umask=022
dirmessage_enable=YES
xferlog_enable=YES
connect_from_port_20=YES
syslog_enable=YES
xferlog_std_format=YES
listen=YES

pam_service_name=vsftpd
userlist_enable=YES
tcp_wrappers=YES
listen_port=2001
listen_address=127.0.0.1
chroot_local_user=YES
allow_writeable_chroot=YES
"""
    with open('/etc/vsftpd/vsftpd.conf', 'w') as out:
        out.write(vsftp_config)

    user_list_config = """root
bin
daemon
adm
lp
sync
shutdown
halt
mail
news
uucp
operator
games
nobody
smestorage
smeconfiguser
"""

    with open('/etc/vsftpd.user_list', 'w') as out:
        out.write(user_list_config)

    check_call(['systemctl', 'restart', 'vsftpd'])
    set_state('sme.vsftpd_init')


@when('sme.vsftpd_init')
@when_not('sme.phpfpm_config')
def php_fpm_config():
    # php-fpm config file
    #
    status_set('maintenance', 'Configuring php-fpm')
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/appliancebuild/conf/php-fpm.d_www.conf', '-O', '/etc/php/5.6/fpm/pool.d/www.conf'])

    ## php.ini configuration

    for line in fileinput.input('/etc/php/5.6/fpm/php.ini', inplace=True):
        # short tags
        if line.strip().startswith('short_open_tag ='):
            line = 'short_open_tag = On'
        # expose php
        if line.strip().startswith('expose_php = '):
            line = 'expose_php = Off'
        # memory limit
        if line.strip().startswith('memory_limit = '):
            line = 'memory_limit = 150M'
        # post max sie
        if line.strip().startswith('post_max_size = '):
            line = 'post_max_size = 300M'
        # upload max size
        if line.strip().startswith('upload_max_filesize = '):
            line = 'upload_max_filesize = 300M'
        # session http only
        if line.strip().startswith('cookie_httponly = '):
            line = 'cookie_httponly = True'
        sys.stdout.write(line)

    for line in fileinput.input('/etc/php/5.6/cli/php.ini', inplace=True):
        # short tags
        if line.strip().startswith('short_open_tag ='):
            line = 'short_open_tag = On'
        # expose php
        if line.strip().startswith('expose_php = '):
            line = 'expose_php = Off'
        # memory limit
        if line.strip().startswith('memory_limit = '):
            line = 'memory_limit = 150M'
        # post max sie
        if line.strip().startswith('post_max_size = '):
            line = 'post_max_size = 300M'
        # upload max size
        if line.strip().startswith('upload_max_filesize = '):
            line = 'upload_max_filesize = 300M'
        # session http only
        if line.strip().startswith('cookie_httponly = '):
            line = 'cookie_httponly = True'
        sys.stdout.write(line)


    set_state('sme.phpfpm_config')

@when('sme.phpfpm_config')
@when_not('sme.boot_services')
def enable_ftp_services_at_boot():
    status_set('maintenance', 'Enabling services at boot')
    check_call(['systemctl', 'enable', 'cloudftp'])
    status_set('maintenance', 'Services enabled')
    set_state('sme.boot_services')

@when('sme.boot_services')
@when_not('sme.ioncube_conf')
def install_ioncube():
    # ioncube
    check_call(['wget', '-q', 'http://downloads3.ioncube.com/loader_downloads/ioncube_loaders_lin_x86-64_5.1.2.tar.gz', '-P', '/tmp/'])
    check_call(['tar', 'zxf', '/tmp/ioncube_loaders_lin_x86-64_5.1.2.tar.gz', '-C', '/tmp'])
    check_call(['cp', '/tmp/ioncube/ioncube_loader_lin_5.6.so', '/usr/lib/php/20131226/ioncube_loader_lin_5.6.so'])
    check_call(['rm', '-rf', '/tmp/ioncube'])
    check_call(['rm', '-rf', '/tmp/ioncube_loaders_lin_x86-64_5.1.2.tar.gz'])

    php_ioncube_config = "zend_extension = /usr/lib/php/20131226/ioncube_loader_lin_5.6.so"

    with open('/etc/php/5.6/mods-available/10-ioncube.ini', 'w') as out:
        out.write(php_ioncube_config)

    os.symlink('/etc/php/5.6/mods-available/10-ioncube.ini', '/etc/php/5.6/cli/conf.d/10-ioncube.ini')
    os.symlink('/etc/php/5.6/mods-available/10-ioncube.ini', '/etc/php/5.6/fpm/conf.d/10-ioncube.ini')

    check_call(['service', 'php5.6-fpm', 'restart'])
    set_state('sme.ioncube_conf')

## ADD JSON_XS
@when('sme.ioncube_conf')
@when_not('sme.install_sme')
def install_sme():
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/codedeploy/1705.01.2/public_html.tar.gz', '-P', '/tmp'])
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/codedeploy/1705.01.2/cron.tar.gz', '-P', '/tmp'])
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/codedeploy/1705.01.2/patches.tar.gz', '-P', '/tmp'])
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/codedeploy/1705.01.2/webdav_html.tar.gz', '-P', '/tmp'])
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/codedeploy/1705.01.2/s3_html.tar.gz', '-P', '/tmp'])
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/codedeploy/1705.01.2/ftpserver.tar.gz', '-P', '/tmp'])
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/appliancebuild/conf/config.tar.gz', '-P', '/tmp'])
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/appliancebuild/conf/upgrade.tar.gz', '-P', '/tmp'])
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/appliancebuild/static/3rdPartyLicensesforOSS.pdf', '-O', '/Third Party Licenses including Open Source Software.pdf'])
    #os.mkdir('/var/www/smestorage')
    check_call(['tar', 'zxf', '/tmp/public_html.tar.gz', '-C', '/var/www/smestorage'])
    check_call(['tar', 'zxf', '/tmp/cron.tar.gz', '-C', '/var/www/smestorage'])
    check_call(['tar', 'zxf', '/tmp/patches.tar.gz', '-C', '/var/www/smestorage'])
    check_call(['tar', 'zxf', '/tmp/webdav_html.tar.gz', '-C', '/var/www/smestorage'])
    check_call(['tar', 'zxf', '/tmp/s3_html.tar.gz', '-C', '/var/www/smestorage'])
    check_call(['tar', 'zxf', '/tmp/ftpserver.tar.gz', '-C', '/var/www/smestorage'])
    check_call(['tar', 'zxf', '/tmp/config.tar.gz', '-C', '/var/www/smestorage'])
    check_call(['tar', 'zxf', '/tmp/upgrade.tar.gz', '-C', '/var/www/smestorage'])

    sitelogs = [
    'sitelogs',
    'sitelogs/amazon',
    'sitelogs/downloads',
    'sitelogs/emailbackup',
    'sitelogs/export',
    'sitelogs/gsites',
    'sitelogs/backup',
    'sitelogs/gdocs',
    'sitelogs/import',
    'sitelogs/processes',
    'sitelogs/boxnet',
    'sitelogs/dropbox',
    'sitelogs/general_tmp',
    'sitelogs/general_tmp/smarty',
    'sitelogs/general_tmp/smarty/cache',
    'sitelogs/general_tmp/smarty/configs',
    'sitelogs/general_tmp/smarty/templates_c',
    'sitelogs/livemesh',
    'sitelogs/skydrive',
    'sitelogs/temptokens',
    'sitelogs/pogoplug',
    'sitelogs/livemesh',
    'sitelogs/reports' ]

    for path in sitelogs:
        os.makedirs('/var/www/smestorage/'+path, exist_ok=True)

    check_call(['touch', '/var/www/smestorage/sitelogs/errorlogs_trace.txt', '/var/www/smestorage/sitelogs/errorlogs.txt'])

    # SME-3608: Patch for perl version
    check_call(['sed', '-i', '-e', 's/defined(@QUERY_PARAM)/@QUERY_PARAM/','/var/www/smestorage/public_html/cgi-bin/uploader/CGI.pm'])

    check_call(['chown', '-R', 'smestorage:smestorage', '/var/www/smestorage'])

    check_call(['rm', '-f','/tmp/*.tar.gz'])
    check_call(['service', 'apache2', 'restart'])
    ###
    open_port(80)
    open_port(443)
    set_state('sme.install_sme')
    set_state('sme.installed')

@when('sme.install_sme')
@when_not('sme.json_xs')
def json_xs():
    os.mkdir('/var/www/smestorage/webdav_html/cgi-bin/JSON_XS')
    check_call(['wget', '-q', 'http://search.cpan.org/CPAN/authors/id/M/ML/MLEHMANN/JSON-XS-3.01.tar.gz', '-P', '/var/www/smestorage/webdav_html/cgi-bin/JSON_XS/'])
    check_call(['wget', '-q', 'http://search.cpan.org/CPAN/authors/id/M/ML/MLEHMANN/Types-Serialiser-1.0.tar.gz', '-P', '/var/www/smestorage/webdav_html/cgi-bin/'])
    check_call(['tar', 'zxf', '/var/www/smestorage/webdav_html/cgi-bin/JSON_XS/JSON-XS-3.01.tar.gz', '-C', '/var/www/smestorage/webdav_html/cgi-bin/JSON_XS/'])
    check_call(['tar', 'zxf', '/var/www/smestorage/webdav_html/cgi-bin/Types-Serialiser-1.0.tar.gz', '-C', '/var/www/smestorage/webdav_html/cgi-bin/'])
    check_call(['perl', 'Makefile.PL', 'INSTALL_BASE=../', ';', 'make;', 'make', 'install'], cwd='/var/www/smestorage/webdav_html/cgi-bin/JSON_XS/JSON-XS-3.01/')
    check_call(['perl', 'Makefile.PL', 'INSTALL_BASE=../', ';', 'make;', 'make', 'install'], cwd='/var/www/smestorage/webdav_html/cgi-bin/Types-Serialiser-1.0/')

    set_state('sme.json_xs')
##

@when('sme.installed', 'leadership.is_leader')
@when_not('sme.crontask_installed')
def crontask_configure():
    crontask = """
MAILTO=''
* * * * * /var/www/smestorage/cron/scheduler_minute.pl
* * * * * /var/www/smestorage/cron/taskmanager.pl
5 4 * * * /var/www/smestorage/cron/scheduler_daily.pl
5 * * * * /var/www/smestorage/cron/scheduler_hour.pl
10 2 1 * * /var/www/smestorage/cron/scheduler_monthly.pl
10 * * * * /var/www/smestorage/cron/scheduler.pl
30 1 * * * /var/www/smestorage/cron/logroller.pl
"""
    with open('/var/spool/cron/crontabs/smestorage', 'w') as cronfile:
        cronfile.write(crontask)
    os.chmod('/var/spool/cron/crontabs/smestorage', 0o600)
    set_state('sme.crontask_installed')

@when('sme.ioncube_conf')
@only_once
def download_sql():
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/appliancebuild/sql/v1705.01-2.seedDB.v4EncUsers.sql', '-O', '/tmp/db.sql'])
    check_call(['wget', '-q', 'http://download.storagemadeeasy.com/appliancebuild/sql/clearDB1705.sql', '-O', '/tmp/clearDB.sql'])
    set_state('sme.sql_downloaded')

@when('sme.sql_downloaded')
@when_not('database.connected')
def missing_mysql():
    status_set('blocked', 'Please link to a database: juju add-relation mariadb sme')

@when('database.connected')
@when_not('database.available')
def waiting_mysql(mysql):
    status_set('waiting', 'Waiting for database to be available')

@when('database.available', 'sme.sql_downloaded', 'leadership.is_leader')
@when_not('sme.sql_filled')
@only_once
def init_db_seed_file(mysql):

    if (not os.path.exists('/tmp/db.sql')): # May have been lost on reboot
        download_sql()

    commandClearSQL = 'mysql -h %s -u %s -p%s %s < /tmp/clearDB.sql' % (mysql.host(),mysql.user(),mysql.password(),mysql.database())
    check_call(commandClearSQL, shell=True)
    command = 'mysql -h %s -u %s -p%s %s < /tmp/db.sql' % (mysql.host(),mysql.user(),mysql.password(),mysql.database())
    check_call(command, shell=True)
    set_state('sme.sql_filled')

## database-relation-changed

@when('database.available')
@when_not('sme.mysql_ready')
def update_config(mysql):
    for line in fileinput.input('/var/www/smestorage/config/public_html/config.inc.php', inplace=True):
        if line.lstrip().startswith('var $dbuser='):
            line = '\tvar $dbuser=\'{}\';\n'.format(mysql.user())
        if line.lstrip().startswith('var $dbpassword='):
            line = '\tvar $dbpassword=\'{}\';\n'.format(mysql.password())
        if line.lstrip().startswith('var $dbname='):
            line = '\tvar $dbname=\'{}\';\n'.format(mysql.database())
        if line.lstrip().startswith('var $dbhost='):
            line = '\tvar $dbhost=\'{}\';\n'.format(mysql.host())
        sys.stdout.write(line)

    if os.path.exists('/var/www/smestorage/public_html/config.inc.php'):
        os.remove('/var/www/smestorage/public_html/config.inc.php')
    os.link('/var/www/smestorage/config/public_html/config.inc.php', '/var/www/smestorage/public_html/config.inc.php')

    if os.path.exists('/var/www/smestorage/public_html/license.txt'):
        os.remove('/var/www/smestorage/public_html/license.txt')
    os.link('/var/www/smestorage/config/public_html/license.txt', '/var/www/smestorage/public_html/license.txt')

    if os.path.exists('/var/www/smestorage/ftpserver/ftpserver.conf'):
        os.remove('/var/www/smestorage/ftpserver/ftpserver.conf')
    os.link('/var/www/smestorage/config/ftpserver/ftpserver.conf', '/var/www/smestorage/ftpserver/ftpserver.conf')

    if os.path.exists('/var/www/smestorage/ftpserver/sftpserver/sftpserver.conf'):
        os.remove('/var/www/smestorage/ftpserver/sftpserver/sftpserver.conf')
    os.link('/var/www/smestorage/config/ftpserver/sftpserver/sftpserver.conf', '/var/www/smestorage/ftpserver/sftpserver/sftpserver.conf')

    if os.path.exists('/var/www/smestorage/webdav_html/cgi-bin/configuration'):
        os.remove('/var/www/smestorage/webdav_html/cgi-bin/configuration')
    os.link('/var/www/smestorage/config/webdav_html/configuration', '/var/www/smestorage/webdav_html/cgi-bin/configuration')

    if os.path.exists('/var/www/smestorage/s3_html/cgi-bin/configuration'):
        os.remove('/var/www/smestorage/s3_html/cgi-bin/configuration')
    os.link('/var/www/smestorage/config/s3_html/configuration', '/var/www/smestorage/s3_html/cgi-bin/configuration')

    if os.path.exists('/var/www/smestorage/cron/config_logroller.conf'):
        os.remove('/var/www/smestorage/cron/config_logroller.conf')
    os.link('/var/www/smestorage/config/cron/config_logroller.conf', '/var/www/smestorage/cron/config_logroller.conf')

    if os.path.exists('/var/www/smestorage/cron/config.conf'):
        os.remove('/var/www/smestorage/cron/config.conf')
    os.link('/var/www/smestorage/config/cron/config.conf', '/var/www/smestorage/cron/config.conf')


    os.chmod('/var/www/smestorage/config/public_html/config.inc.php', 0o640)
    check_call(['chown', '-R', 'smestorage:smestorage', '/var/www/smestorage/'])
    call(['php', '/var/www/smestorage/upgrade/dbupgrade.php'])
    set_state('sme.mysql_ready')
    status_set('active', 'ready')

hooks = Hooks()

@hooks.hook('update-status')
def update_status():
    pass
